    
  -- extensions: put here the libraries to be available to the scripts.
  -- a library can be a lua file (loaded with <cgilua.dofile>) or
  -- a dynamic library (loaded with <loadlibrary>).
  -- (see also the function loadlibrary in the documentation).
  
  -- mail library 
  cgilua.dofile( 'mail.lua' )
  
  -- HTML toolkit
  -- cgilua.dofile( 'htmltoolkit.lua' )

  -- HTK library
  -- cgilua.dofile( 'htk.lua' )

  -- cookies library
  cgilua.dofile( 'cookies.lua' )

  -- state managing library
  cgilua.dofile( 'state.lua' )
  
  -- date library
  cgilua.loadlibrary( 'date', 'datelib_open' )
   
  -- auxiliar library
  cgilua.loadlibrary( 'auxiliar', 'auxiliarlib_open' )

  -- database library
  -- cgilua.dofile( 'dblua.lua' )

  -- library for criptography
  -- cgilua.loadlibrary( 'crypt', 'cryptlib_open' )
  
  -- security definitions: here some dangerous functions are
  -- removed or redefined to guarantee the safety of the scripts.
  -- the definitions depends on the security level required.
  
  -- the function <loadlib> is intended to be used 
  -- by the function cgilua.loadlibrary only.
  -- to keep it available to the scripts represents
  -- that any script can execute any C function (anything).
  
  cgilua.loadlibrary        = nil
  loadlib                   = nil
  unloadlib                 = nil
  callfromlib               = nil
  
  
  -- the function <capture> is an example of how a function
  -- can be implemented to capture the results (I/O displayed)
  -- of an external program (parameter cmd). This function is 
  -- cleary unsecure, it is recommended to use it always
  -- in a more restrictive implementation. Note the use
  -- of the upvalue %readfrom to use the original readfrom()
  
  --function capture ( cmd )
  --  local lcreadfrom = %readfrom
  --  lcreadfrom('|' .. cmd )
  --  local result  = read( '.*' )
  --  lcreadfrom()
  --  return result
  --end


  -- the original function <execute> let shell comands be invoked.
  -- it is redefined to act like every command execution fails,
  -- keeping it compatible with the standard definition.
  
  function execute( )
    return -1
  end
  
  
  -- the functions <writeto> and <readfrom>
  -- accepts a parameter starting with a '|';
  -- in this case the input/output is connected to a shell command
  -- via a pipe. The redefined versions fails if the parameter 
  -- starts with a '|' to prevent shell commands to be invoked.

  -- redefinition of <writeto>
  
  function writeto( file )
    if type(file) == 'string' and strsub(file, 1, 1) == '|' then
      return nil
    end
    local piped = %writeto
    if file then
      return piped( file )
    else
      return piped()
    end
  end
  
  -- redefinition of <readfrom>
  
  function readfrom( file )
    if type(file) == 'string' and strsub(file, 1, 1) == '|' then
      return nil
    end
    local piped = %readfrom
    if file then
      return piped( file )
    else
      return piped()
    end
  end

  -- the function <myerror> is an example of how the error method can be 
  -- redefined to send a message to the webmaster with the error description
  -- instead of the default behavior of showing the error
  -- to the user.
  -- 
  -- function myerror( msg )
  --   local mailmsg = "" 
  -- 
  --   local url = format( 'http://%s%s%s', getenv("SERVER_NAME"),
  --               getenv("SCRIPT_NAME"), getenv("PATH_INFO") )
  --   local qs =  getenv("QUERY_STRING")
  --   if (qs) then url = url .. "?" .. qs end
  --
  --   mailmsg = mailmsg .. format( "URL: %s\n", url )
  --   mailmsg = mailmsg .. format( "cgi table: %s\n", cgilua.tostring( cgi ) )
  --   mailmsg = mailmsg .. format( "File: %s\n", cgilua.script_path )
  --   mailmsg = mailmsg .. format( "Error: %s\n", msg )
  --   mailmsg = mailmsg .. format( "\nStack:\n" )
  --
  --   local i = 1
  --   local s = cgilua.getstack()
  --   while s[i] do
  --     mailmsg = mailmsg .. s[i] .. "\n"
  --     i = i + 1
  --   end
  --
  --   mail{ from='webmaster@domain.com", to='webmaster@domain.com', subject='Error in script', message=mailmsg, mailserver = "127.0.0.1" }
  -- end
  --
  -- cgilua.seterrormethod( myerror )

    
  -- definitions for compatibility with previous versions
    
  -- version 2.0
  cgilua.relative_url = cgilua.relativeurl
    
    
